@@ -3,6 +3,7 @@
 # Copyright (c) 2019 Red Hat, Inc.
 # Copyright (c) 2019 Tomáš Mráz <tmraz@fedoraproject.org>
 
+import re
 from subprocess import CalledProcessError, check_output
 
 from .configgenerator import ConfigGenerator
@@ -23,7 +24,7 @@ activate = 1
 
 
 class OpenSSLGenerator(ConfigGenerator):
-    CONFIG_NAME = 'openssl'
+    CONFIG_NAME = 'opensslcnf'
 
     cipher_not_map = {
         'AES-256-CTR': '',
@@ -115,114 +116,17 @@ class OpenSSLGenerator(ConfigGenerator):
         'TLS_SHA256_SHA256': {
             'cipher': {'NULL'},
             'hash': {'SHA2-256'},
-            'mac': {'SHA2-256'},
+            'mac': {'HMAC-SHA2-256'},
             'protocol': {'TLS1.3', 'DTLS1.3'},
         },
-        'TLS_SHA256_SHA384': {
+        'TLS_SHA384_SHA384': {
             'cipher': {'NULL'},
             'hash': {'SHA2-384'},
-            'mac': {'SHA2-384'},
+            'mac': {'HMAC-SHA2-384'},
             'protocol': {'TLS1.3', 'DTLS1.3'},
         },
     }
 
-    @classmethod
-    def generate_ciphers(cls, policy):
-        s = ''
-        p = policy.enabled
-        ip = policy.disabled
-        # We cannot separate RSA strength from DH params.
-        min_dh_size = policy.integers['min_dh_size']
-        min_rsa_size = policy.integers['min_rsa_size']
-        if min_dh_size < 1023 or min_rsa_size < 1023:
-            s = cls.append(s, '@SECLEVEL=0')
-        elif min_dh_size < 2048 or min_rsa_size < 2048:
-            s = cls.append(s, '@SECLEVEL=1')
-        elif min_dh_size < 3072 or min_rsa_size < 3072:
-            s = cls.append(s, '@SECLEVEL=2')
-        else:
-            s = cls.append(s, '@SECLEVEL=3')
-
-        for i in p['key_exchange']:
-            try:
-                s = cls.append(s, cls.key_exchange_map[i])
-            except KeyError:
-                pass
-
-        for i in ip['key_exchange']:
-            try:
-                s = cls.append(s, cls.key_exchange_not_map[i])
-            except KeyError:
-                pass
-
-        for i in ip['cipher']:
-            try:
-                s = cls.append(s, cls.cipher_not_map[i])
-            except KeyError:
-                pass
-        for keyword, cipherset in cls.cipher_notany_multimap.items():
-            if all(c in ip['cipher'] for c in cipherset):
-                s = cls.append(s, keyword)
-
-        for i in ip['mac']:
-            try:
-                s = cls.append(s, cls.mac_not_map[i])
-            except KeyError:
-                pass
-
-        # These ciphers are not necessary for any
-        # policy level, and only increase the attack surface.
-        # FIXME! must be fixed for custom policies
-        for c in ('-SHA384', '-CAMELLIA', '-ARIA', '-AESCCM8'):
-            s = cls.append(s, c)
-
-        return s
-
-    @classmethod
-    def generate_ciphersuites(cls, policy):
-        s = ''
-        p = policy.enabled
-        # we need the output sorted by order of `cipher`
-        # finer sorting nuances are currently ignored
-        for c in p['cipher']:
-            cipher_submap = {name: spec for name, spec
-                             in cls.ciphersuite_map.items()
-                             if spec['cipher'] == {c}}
-            for ciphersuite_name, ciphersuite_spec in cipher_submap.items():
-                if all(any(val in algvalues for val in p[algclass])
-                       for algclass, algvalues in ciphersuite_spec.items()):
-                    s = cls.append(s, ciphersuite_name)
-        return s
-
-    @classmethod
-    def generate_config(cls, unscoped_policy):
-        policy = unscoped_policy.scoped({'tls', 'ssl', 'openssl'})
-        return cls.generate_ciphers(policy) + '\n'
-
-    @classmethod
-    def test_config(cls, config):
-        output = b''
-        assert config.endswith('\n')  # noqa: S101
-        try:
-            output = check_output(['openssl',  # noqa: S607
-                                   'ciphers', config[:-1]])
-        except CalledProcessError:
-            cls.eprint('There is an error in openssl generated policy')
-            cls.eprint(f'Policy:\n{config}')
-            return False
-        except OSError:
-            # Ignore missing openssl
-            return True
-        if b'NULL' in output or b'ADH' in output:
-            cls.eprint('There is NULL or ADH in openssl generated policy')
-            cls.eprint(f'Policy:\n{config}')
-            return False
-        return True
-
-
-class OpenSSLConfigGenerator(OpenSSLGenerator):
-    CONFIG_NAME = 'opensslcnf'
-
     # has to cover everything c-p has
     protocol_map = {
         'SSL3.0': 'SSLv3',
@@ -332,13 +236,81 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
         'MLKEM768': '?mlkem768',
         'P384-MLKEM768': '?p384_mlkem768',
         'X448-MLKEM768': '?x448_mlkem768',
-        'X25519-MLKEM768': '?X25519MLKEM768:?x25519_mlkem768',  # old/new name
-        'P256-MLKEM768': '?SecP256r1MLKEM768:?p256_mlkem768',  # old/new name
+        'X25519-MLKEM768': '?X25519MLKEM768:?x25519_mlkem768',  # new/old name
+        'P256-MLKEM768': '?SecP256r1MLKEM768:?p256_mlkem768',  # new/old name
         'MLKEM1024': '?mlkem1024',
         'P521-MLKEM1024': '?p521_mlkem1024',
-        'P384-MLKEM1024': '?p384_mlkem1024',
+        'P384-MLKEM1024': '?SecP384r1MLKEM1024:?p384_mlkem1024',  # new/old
     }
 
+    @classmethod
+    def generate_ciphers(cls, policy):
+        s = ''
+        p = policy.enabled
+        ip = policy.disabled
+        # We cannot separate RSA strength from DH params.
+        min_dh_size = policy.integers['min_dh_size']
+        min_rsa_size = policy.integers['min_rsa_size']
+        if min_dh_size < 1023 or min_rsa_size < 1023:
+            s = cls.append(s, '@SECLEVEL=0')
+        elif min_dh_size < 2048 or min_rsa_size < 2048:
+            s = cls.append(s, '@SECLEVEL=1')
+        elif min_dh_size < 3072 or min_rsa_size < 3072:
+            s = cls.append(s, '@SECLEVEL=2')
+        else:
+            s = cls.append(s, '@SECLEVEL=3')
+
+        for i in p['key_exchange']:
+            try:
+                s = cls.append(s, cls.key_exchange_map[i])
+            except KeyError:
+                pass
+
+        for i in ip['key_exchange']:
+            try:
+                s = cls.append(s, cls.key_exchange_not_map[i])
+            except KeyError:
+                pass
+
+        for i in ip['cipher']:
+            try:
+                s = cls.append(s, cls.cipher_not_map[i])
+            except KeyError:
+                pass
+        for keyword, cipherset in cls.cipher_notany_multimap.items():
+            if all(c in ip['cipher'] for c in cipherset):
+                s = cls.append(s, keyword)
+
+        for i in ip['mac']:
+            try:
+                s = cls.append(s, cls.mac_not_map[i])
+            except KeyError:
+                pass
+
+        # These ciphers are not necessary for any
+        # policy level, and only increase the attack surface.
+        # FIXME! must be fixed for custom policies
+        for c in ('-SHA384', '-CAMELLIA', '-ARIA', '-AESCCM8'):
+            s = cls.append(s, c)
+
+        return s
+
+    @classmethod
+    def generate_ciphersuites(cls, policy):
+        s = ''
+        p = policy.enabled
+        # we need the output sorted by order of `cipher`
+        # finer sorting nuances are currently ignored
+        for c in p['cipher']:
+            cipher_submap = {name: spec for name, spec
+                             in cls.ciphersuite_map.items()
+                             if spec['cipher'] == {c}}
+            for ciphersuite_name, ciphersuite_spec in cipher_submap.items():
+                if all(any(val in algvalues for val in p[algclass])
+                       for algclass, algvalues in ciphersuite_spec.items()):
+                    s = cls.append(s, ciphersuite_name)
+        return s
+
     @classmethod
     def generate_config(cls, unscoped_policy):
         policy = unscoped_policy.scoped({'tls', 'ssl', 'openssl'})
@@ -369,13 +341,35 @@ class OpenSSLConfigGenerator(OpenSSLGenerator):
         if policy.enums['__ems'] == 'RELAX':
             s += 'Options = RHNoEnforceEMSinFIPS\n'
 
+        # default size for req key generation,
+        # to be consumed when bz2349857 gets fixed
+        default_rsa_size = max(policy.integers['min_rsa_size'], 2048)
+        s += f'\n[req]\ndefault_bits = {default_rsa_size}\n'
+
         if 'SHA1' in p['hash']:
             s += RH_ALLOW_SHA1
 
         return s
 
     @classmethod
-    def test_config(cls, config):  # pylint: disable=unused-argument
+    def test_config(cls, config):
+        output = b''
+        ciphers, = re.findall(r'^CipherString = (.*)$', config, re.MULTILINE)
+
+        try:
+            output = check_output(['openssl',  # noqa: S607
+                                   'ciphers', ciphers])
+        except CalledProcessError:
+            cls.eprint('There is an error in openssl generated policy')
+            cls.eprint(f'Policy:\n{config}')
+            return False
+        except OSError:
+            # Ignore missing openssl
+            return True
+        if b'ADH' in output:
+            cls.eprint('There is ADH in openssl generated policy')
+            cls.eprint(f'Policy:\n{config}')
+            return False
         return True
 
 
